#include "taskFlyport.h"
#import "RS485Helper.h"
#import "RS232Helper.h"
#import "cJSON.h"

#define		DE_485		p2
#define		RE_485		p17
#define		TX_485		p5
#define		RX_485		p7

#define		TX_232		p4
#define		RX_232		p6
#define		CTS_232		p11
#define		RTS_232		p9

const int port485 = 	2;
const int port232 = 	3;

void FlyportTask()
{
	// Expansion Connector Pins
	IOInit(p1, out);		// pin 3
	IOInit(p3, out);		// pin 4
	IOInit(p8, out);		// pin 5
	IOInit(p10, out);		// pin 6
	IOPut(p1, off);
	IOPut(p3, off);
	IOPut(p8, off);
	IOPut(p10, off);
	IOInit(p12, indown);	// pin 7
	IOInit(p14, indown);	// pin 8
	
	// Initialize the RS485
	RS485Off(port485);
	RS485Remap(port485, TX_485, RX_485, DE_485, RE_485);
	RS485Init(port485, 19200);
	RS485SetParam(port485, RS485_STOP_BITS, RS485_ONE_STOP);
	RS485SetParam(port485, RS485_DATA_PARITY, RS485_8BITS_PARITY_NONE);
	RS485On(port485);

	// Initialize the RS232

	RS232Off(port232);
	// RS232Remap(port232, TX_232, RX_232, RTS_232, CTS_232);
	RS232Remap(port232, TX_232, RX_232, 0, 0);
	RS232Init(port232, 19200);
	RS232SetParam(port232, RS232_STOP_BITS, RS232_ONE_STOP);
	RS232SetParam(port232, RS232_DATA_PARITY, RS232_8BITS_PARITY_NONE);
	RS232On(port232);

	vTaskDelay(50);

	
	// Flyport connects to default network
	WFConnect(WF_DEFAULT);
	while(WFGetStat() != CONNECTED);
	vTaskDelay(25);
	UARTWrite(1,"Flyport Wi-fi G connected...hello world!\r\n");
	vTaskDelay(200);
	
	char msgBuffer[256];

	TCP_SOCKET serverSocket = INVALID_SOCKET;
	
	while(1)
	{
		//	Main user's firmware loop
	
		// wenn TCP Socket verbunden ist, lassen D5out leuchten
		if (TCPisConn(serverSocket)) {
			IOPut(D5Out, ON);
		} else {
			IOPut (D5Out, OFF);
		}
		
		// Verbindungsaufbau
		if (serverSocket == INVALID_SOCKET) {
			
			// ServerSocket auf Port 9988 anbieten
			UARTWrite(1, "setup serverSocket...\r\n");
			serverSocket = TCPServerOpen("9988");
			
			// Warten, bis das alles ok ist
			UARTWrite(1, "setup serverSocket waiting...\r\n");
			while (serverSocket == INVALID_SOCKET);
			
			UARTWrite(1, "serverSocket ready...\r\n");
		} else {
			// Wir haben also eine Socket? Na, dann wollen wir doch mal versuchen daraus etwas zu lesen
			int rxlen = TCPRxLen (serverSocket);
			if (rxlen > 0 && rxlen < 255) {
				char readBuffer[256];
				TCPRead (serverSocket, readBuffer, rxlen);
				//UARTWrite (1, readBuffer);
				cJSON* json = cJSON_Parse (readBuffer);
				if (!json) {
					sprintf (msgBuffer, "json error: %s\r\n", cJSON_GetErrorPtr());
					UARTWrite (1, msgBuffer);
				} else {
					char* type = cJSON_GetObjectItem (json, "type")->valuestring;
					if (!strcmp (type, "direction")) {
						char* direction = cJSON_GetObjectItem (json, "direction")->valuestring;
						sprintf (msgBuffer, "direction: %s\r\n", direction);
						UARTWrite (1, msgBuffer);
						
						sprintf (msgBuffer, "%c", direction[0]);
						RS485Write (port485, msgBuffer);
					}
				}
				cJSON_Delete (json);
			}
		}
	}
}
