//
//  BrainProducer.m
//  Macoun2013
//
//  Created by Frank Jüstel on 04.10.13.
//  Copyright (c) 2013 JSM Arts Webservices GbR. All rights reserved.
//

#import "BrainProducer.h"

// Für den Hirnschmalzmesser
#import "TGAccessoryDelegate.h"
#import "TGAccessoryManager.h"

@interface BrainProducer () <TGAccessoryDelegate>

@end

@implementation BrainProducer

+(instancetype)sharedInstance
{
	static BrainProducer* sharedInstance;
	static dispatch_once_t onceToken;
	dispatch_once(&onceToken, ^{
		sharedInstance = [[BrainProducer alloc] init];
	});
	return sharedInstance;
}

-(instancetype) init
{
	if (self = [super init]) {
		// Diese Werte sollen vom ThinkGear geholt werden
		self.listOfPostedThinkGearValuesIfChanged = @[@"eSenseAttention", @"eSenseMeditation", @"blinkStrength", @"eegLowAlpha", @"eegHighAlpha", @"eegLowBeta", @"eegHighBeta", @"eegLowGamma", @"eegHighGamma"];
		
		// ThinkGear Einstellungen 5 mal pro Sekunde über HeadSet
		[[TGAccessoryManager sharedTGAccessoryManager] setupManagerWithInterval:0.2 forAccessoryType:TGAccessoryTypeDongle];
		
		// Wir sind der Delegate
		[[TGAccessoryManager sharedTGAccessoryManager] setDelegate:self];
		
		// Keine RawData, sondern "echte"
		[[TGAccessoryManager sharedTGAccessoryManager] setRawEnabled:NO];
		
		// Haben wir schon ein HeadSet dran? Dann gleich den Stream starten
		if ([[TGAccessoryManager sharedTGAccessoryManager] accessory]) {
			[[TGAccessoryManager sharedTGAccessoryManager] startStream];
		}
	}
	
	return self;
}

#pragma mark - TGAccessoryDelegate

// Hurra, ein HeadSet wurde angeschlossen
-(void)accessoryDidConnect:(EAAccessory *)accessory
{
	//	dispatch_async(dispatch_get_main_queue(), ^{
	//		UIAlertView* alert = [[UIAlertView alloc] initWithTitle:@"message:" message:[NSString stringWithFormat:@"ThinkGear accessory %@ did connect", accessory.name] delegate:nil cancelButtonTitle:@"ok" otherButtonTitles:nil];
	//		[alert show];
	//	});
	
	// Datenstrom vom ThinkGear Device starten
	[[TGAccessoryManager sharedTGAccessoryManager] startStream];
}

// Schade, das Headset wurde abgeschaltet oder es ist out of range....
-(void)accessoryDidDisconnect
{
	//	dispatch_async(dispatch_get_main_queue(), ^{
	//		UIAlertView* alert = [[UIAlertView alloc] initWithTitle:@"message:" message:@"ThinkGear accessory disconnected"delegate:nil cancelButtonTitle:@"ok" otherButtonTitles:nil];
	//		[alert show];
	//	});
}

// Wir empfangen Daten vom Headset.
-(void)dataReceived:(NSDictionary *)data
{
	// poorSignal ist so etwas wie die Signalstärke. Wir verwenden es um das entsprechende Statusbild der Verbindung zu zeigen, also werden wir diesen Wert immer Posten, und nicht nur, wenn er sich ändert
	if (data[@"poorSignal"]) {
		[[NSNotificationCenter defaultCenter] postNotificationName:@"poorSignal" object:data[@"poorSignal"]];
	}
	
	// Für alle anderen Werte wollen wir nur eine Notification rausschicken, wenn sich der Wert auch geändert hat.
	static NSMutableDictionary* staticValueCopy;
	if (!staticValueCopy) {
		staticValueCopy = [NSMutableDictionary new];
	}
	
	for (NSString* signal in self.listOfPostedThinkGearValuesIfChanged) {
		if (data[signal] && [data[signal] integerValue] != [staticValueCopy[signal] integerValue]) {
			staticValueCopy[signal] = data[signal];
			[[NSNotificationCenter defaultCenter] postNotificationName:signal object:data[signal]];
		}
	}
}


@end
