//
//  BrainViewController.m
//  Macoun2013
//
//  Created by Frank Jüstel on 02.10.13.
//  Copyright (c) 2013 JSM Arts Webservices GbR. All rights reserved.
//

#import <QuartzCore/QuartzCore.h>

#import "BrainViewController.h"
#import "FRD3DBarChartViewController.h"
#import "RoboTalker.h"
#import "BrainProducer.h"

@interface BrainViewController () <FRD3DBarChartViewControllerDelegate>
@property (weak, nonatomic) IBOutlet UILabel *attentionSliderValueLabelOutlet;
@property (weak, nonatomic) IBOutlet UISlider *attentionSliderOutlet;
@property (weak, nonatomic) IBOutlet UILabel *attentionLabelOutlet;
@property (weak, nonatomic) IBOutlet UIImageView *backgroundImageViewOutlet;
@property (weak, nonatomic) IBOutlet UIImageView *connectionStatusImageOutlet;
@property (weak, nonatomic) IBOutlet UIView *containerViewOutlet;

@property FRD3DBarChartViewController* chartViewController;
@property NSMutableDictionary* thinkGearValues;
@property NSMutableDictionary* thinkGearMaxValues;
@property NSMutableArray* listOfBlockObservers;
@property NSUInteger blinkStrengthCollector;
@property NSTimer* chartUpdateTimer;
@property NSTimer* checkSendCommandTimer;
@end

@implementation BrainViewController

- (IBAction)attentionSliderValueChanged:(UISlider*)slider
{
	self.attentionSliderValueLabelOutlet.text = [NSString stringWithFormat:@"%d", (int) slider.value];
	[[NSUserDefaults standardUserDefaults] setFloat:slider.value forKey:@"attentionSlider"];
	[[NSUserDefaults standardUserDefaults] synchronize];
}

#pragma mark - view lifecycle
- (void)viewDidLoad
{
    [super viewDidLoad];
	
	self.thinkGearValues = [NSMutableDictionary new];
	self.thinkGearMaxValues = [NSMutableDictionary new];
	self.listOfBlockObservers = [NSMutableArray new];
	
	self.backgroundImageViewOutlet.image = [[self.backgroundImageViewOutlet.image blur] scaleIfNeeded];
	
	self.connectionStatusImageOutlet.backgroundColor = [UIColor colorWithRed:1 green:1 blue:1 alpha:0.5];
	self.connectionStatusImageOutlet.layer.cornerRadius = 12.0;
		
	self.containerViewOutlet.layer.cornerRadius = 12.0;
	
	self.attentionLabelOutlet.layer.cornerRadius = 12.0;
	self.attentionLabelOutlet.alpha = 0.5;
	
	self.attentionSliderOutlet.value = [[NSUserDefaults standardUserDefaults] floatForKey:@"attentionSlider"];
	self.attentionSliderValueLabelOutlet.text = [NSString stringWithFormat:@"%d", (int) self.attentionSliderOutlet.value];
	
}

-(void)viewDidAppear:(BOOL)animated
{
	[super viewDidAppear:animated];
	
	NSNotificationCenter* center = [NSNotificationCenter defaultCenter];
	
	// Wenn sich das poorSignal ändern prüfen wir, ob ein neues ConnectionBild gesetzt werden muss
	id observer = [center addObserverForName:@"poorSignal" object:nil queue:nil usingBlock:^(NSNotification *notification) {
		NSInteger signalValue = [notification.object integerValue];
		
		dispatch_async(dispatch_get_main_queue(), ^{
			UIImage* poorSignalValueImage = [self connectionStatusImageForSignalValue: signalValue];
			if (poorSignalValueImage != self.connectionStatusImageOutlet.image) {
				self.connectionStatusImageOutlet.image = poorSignalValueImage;
			}
		});
	}];
	[self.listOfBlockObservers addObject:observer];
	
	for (NSString* signalName in [[BrainProducer sharedInstance] listOfPostedThinkGearValuesIfChanged]) {
		observer = [center addObserverForName:signalName object:nil queue:nil usingBlock:^(NSNotification *notification) {
			self.thinkGearValues[signalName] = notification.object;
			
			// Maximum für jeden einzelnen Wert in dieser Sitzung ermitteln
			if ([self.thinkGearMaxValues[signalName] integerValue] < [notification.object integerValue]) {
				self.thinkGearMaxValues[signalName] = notification.object;
			}
			
			// Wenn ein eSenseAttention kommt, werden wir sofort die Prozentanzeige updaten
			if ([signalName isEqualToString:@"eSenseAttention"]) {
				dispatch_async(dispatch_get_main_queue(), ^{
					self.attentionLabelOutlet.text = [NSString stringWithFormat:@"%d %%", (int) [notification.object integerValue]];
				});
			}
			
			// Wenn ein blinkStrength kommt, zählen wir den Zwinkerzähler hoch
			if ([signalName isEqualToString:@"blinkStrength"]) {
				self.blinkStrengthCollector++;
			}
		}];
		[self.listOfBlockObservers addObject:observer];
	}
	
	self.chartUpdateTimer = [NSTimer scheduledTimerWithTimeInterval:1.0 target:self selector:@selector(updateChart:) userInfo:nil repeats:YES];
	self.checkSendCommandTimer = [NSTimer scheduledTimerWithTimeInterval:1.5 target:self selector:@selector(checkSendCommand:) userInfo:nil repeats:YES];
}

-(void)viewWillDisappear:(BOOL)animated
{
	NSLog(@"%s", __PRETTY_FUNCTION__);
	[super viewWillDisappear:animated];
	
	// Observer wieder abmelden
	for (id observer in self.listOfBlockObservers) {
		[[NSNotificationCenter defaultCenter] removeObserver:observer];
	}
	// Liste löschen, sonst gibt's immer noch referenzen auf die Observer und dealloc wird niemals nicht nie gerufen :-)
	[self.listOfBlockObservers removeAllObjects];
	
	[self.chartUpdateTimer invalidate];
	[self.checkSendCommandTimer invalidate];
	
	self.chartViewController.frd3dBarChartDelegate = nil;
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

-(void)dealloc
{
	NSLog(@"%s", __PRETTY_FUNCTION__);
}

#pragma mark - selectors
-(void) updateChart: (id) sender
{
	dispatch_async(dispatch_get_main_queue(), ^{
		[self.chartViewController updateChartAnimated:YES animationDuration:1.0 options:kUpdateChartOptionsDoNotUpdateLegends];
	});
}

-(void) checkSendCommand: (id) sender
{
	// Wenn der Aufmerksamkeitswert über dem Wert des Sliders?
	if ([self.thinkGearValues[@"eSenseAttention"] integerValue] >= self.attentionSliderOutlet.value) {
        // Dann fahren wir vorwärts :-)
		[[RoboTalker sharedInstance] sendForward];
	}

	if (self.blinkStrengthCollector >= 5) {
        // Wenn wir fünf Zwinker haben, dann geht's links herum
		[[RoboTalker sharedInstance] sendLeft];
	} else if (self.blinkStrengthCollector >= 3) {
        // Wenn wir mehr als drei haben, dann rechts
		[[RoboTalker sharedInstance] sendRight];
	}
	
	// Collector für's Zwinkerzählen zurücksetzen
	self.blinkStrengthCollector = 0;
}

#pragma mark - private methods

// Je nach poorSignaValue ein entsprechendes connection picture ausgeben
- (UIImage *)connectionStatusImageForSignalValue: (NSInteger) signalValue
{
	// Auch wenn [UIImage imageNamed...] sicher clever implementiert ist, wir holen uns die Bilder nur einmal
	static UIImage* connectedImage;
	static UIImage* connectingImage3;
	static UIImage* connectingImage2;
	static UIImage* connectingImage1;
	static UIImage* disconnectedImage;
	
	if (!connectedImage) {
		connectedImage = [UIImage imageNamed:@"Signal_Connected"];
		connectingImage3 = [UIImage imageNamed:@"Signal_Connecting3"];
		connectingImage2 = [UIImage imageNamed:@"Signal_Connecting2"];
		connectingImage1 = [UIImage imageNamed:@"Signal_Connecting1"];
		disconnectedImage = [UIImage imageNamed:@"Signal_Disconnected"];
	}
	
	// Je nach poorSignalValue das entsprechende Bild zurück liefern
	switch (signalValue) {
		case 0:
			return connectedImage;

		case 1:
		case 49:
			return connectingImage3;
			
		case 50:
		case 199:
			return connectingImage2;
			
		case 200:
			return connectingImage1;

		default:
			return disconnectedImage;
	}
}

#pragma mark - segues
-(void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender
{
	if ([segue.identifier isEqualToString:@"chart"]) {
		self.chartViewController = segue.destinationViewController;
		self.chartViewController.frd3dBarChartDelegate = self;
		self.chartViewController.useCylinders = YES;
	}
}

-(void) update: (id)sender
{
	[self.chartViewController reloadInputViews];
}

#pragma mark - FRD3DBarChartViewControllerDelegate
// Anzahl der Zeilen im Chart
-(int) frd3DBarChartViewControllerNumberRows:(FRD3DBarChartViewController *) frd3DBarChartViewController
{
	return 1;
}

// Anzahl der Spalten im Chart
-(int) frd3DBarChartViewControllerNumberColumns:(FRD3DBarChartViewController *) frd3DBarChartViewController
{
	return (int)[BrainProducer sharedInstance].listOfPostedThinkGearValuesIfChanged.count;
}

// Maximalwert der erreicht werden soll
-(float) frd3DBarChartViewControllerMaxValue:(FRD3DBarChartViewController *) frd3DBarChartViewController
{
	return 100.0;
}

// Höhe eines einzelnen Wertes
-(float) frd3DBarChartViewController:(FRD3DBarChartViewController *)frd3DBarChartViewController valueForBarAtRow:(int)row column:(int)column
{
	NSString* valueName = [BrainProducer sharedInstance].listOfPostedThinkGearValuesIfChanged[column];
	CGFloat floatValue = [self.thinkGearValues[valueName] floatValue];
	
	// Wenn wir eegWerte haben, nehmen wir maxEEGValue * 2, damit sich alle unterhalb der 50% Marke darstellen
	if ([valueName hasPrefix:@"eeg"]) {
		CGFloat maxFloatValue = [self.thinkGearMaxValues[valueName] floatValue];
		floatValue = maxFloatValue ? 100.0 / maxFloatValue * floatValue : 0;
	}
	return floatValue;
}

// Legende für die einzelnen Spalten
-(NSString *) frd3DBarChartViewController:(FRD3DBarChartViewController *)frd3DBarChartViewController legendForColumn:(int) column
{
	NSString* valueName = [BrainProducer sharedInstance].listOfPostedThinkGearValuesIfChanged[column];
	return valueName;
}

// Unterteilung der "Gitterwand"
-(int) frd3DBarChartViewControllerNumberHeightLines:(FRD3DBarChartViewController *)frd3DBarChartViewController
{
	return 10;
}

// Beschriftung der "Gitterwand"
-(NSString *) frd3DBarChartViewController:(FRD3DBarChartViewController *)frd3DBarChartViewController legendForValueLine:(int) line
{
	return [NSString stringWithFormat:@"%d", (line + 1) * (100 / [self frd3DBarChartViewControllerNumberHeightLines:frd3DBarChartViewController])];
}

// Farbe der Säulen
-(UIColor *) frd3DBarChartViewController:(FRD3DBarChartViewController *)frd3DBarChartViewController colorForBarAtRow:(int)row column:(int) column
{
	NSString* signalName = [BrainProducer sharedInstance].listOfPostedThinkGearValuesIfChanged[column];
	CGFloat floatValue = [self.thinkGearValues[signalName] floatValue];
	
	if ([signalName hasPrefix:@"eeg"]) {
		CGFloat maxFloatValue = [self.thinkGearMaxValues[signalName] floatValue];
		floatValue = maxFloatValue ? 100.0 / maxFloatValue * floatValue: 0;
	}
	
	CGFloat floatMax = [self frd3DBarChartViewControllerMaxValue:frd3DBarChartViewController];
	return [UIColor colorWithRed:floatValue / floatMax green:1.0 - floatValue / floatMax blue:1.0 alpha:1.0];
}
@end
